@@ -1,4 +1,4 @@
-#!/usr/bin/env python3.6
+#!/usr/bin/env python3.7
 
 # Main script for sequencing analysis
 # Input of this script:
@@ -25,13 +25,18 @@ import argparse
 import logging
 import datetime
 import shutil
-# import time
+import time
 
 # pakage modules
 from TileSeqMut import help_functions
 from TileSeqMut import count_mut
 from TileSeqMut import cluster
 
+# pakage modules
+# import help_functions
+# import count_mut
+# import cluster
+
 class fastq2counts(object):
     """Main class to read fastq files and output mutation counts."""
 
@@ -147,15 +152,20 @@ class fastq2counts(object):
             # wait for alignment to finish and call mutations
 
         elif self._args.environment == "BC2" or self._args.environment == "DC" or self._args.environment == "BC":
-            # make sh files to submit to BC
-            sh_output = os.path.join(self._output, "BC_aln_sh")
-            os.system("mkdir "+sh_output)
-
             if self._args.environment == "BC2" or self._args.environment == "BC":
+                # make sh files to submit to BC
+                sh_output = os.path.join(self._output, "BC_aln_sh")
+                os.system("mkdir " + sh_output)
                 # make sh files to submit to BC
                 self._log.info("Submitting alignment jobs to BC/BC2...")
                 # alignment_sh_bc2(fastq_map, ref_name, ref_seq, ref_path, sam_path, sh_output, at, logging)
-                sam_df, job_list = cluster.alignment_sh_bc2(fastq_map, self._project, self._seq.seq.values.item(), ref_path, sam_output, sh_output, self._args.at, self._log)
+                if  self._args.rc:
+                    rc = True
+                else:
+                    rc = False
+                sam_df, job_list = cluster.alignment_sh_bc2(fastq_map, self._project, self._seq.seq.values.item(),
+                                                            ref_path, sam_output, sh_output, self._args.at,
+                                                            self._log, rc)
                 self._log.info("Alignment jobs are submitte to BC2. Check pbs-output for STDOUT/STDERR")
 
             elif self._args.environment == "DC":
@@ -164,13 +174,23 @@ class fastq2counts(object):
                 os.system("mkdir "+sh_output)
 
                 self._log.info("Submitting alignment jobs to DC...")
-                sam_df, job_list = cluster.alignment_sh_dc(fastq_map, self._project, self._seq.seq.values.item(), ref_path, sam_output, sh_output, self._args.at, self._log)
+                if  self._args.rc:
+                    rc = True
+                else:
+                    rc = False
+                sam_df, job_list = cluster.alignment_sh_dc(fastq_map, self._project, self._seq.seq.values.item(),
+                                                           ref_path, sam_output, sh_output,
+                                                           self._args.at, self._log, rc)
                 self._log.info("Alignment jobs are submitte to DC. Check pbs-output for STDOUT/STDERR")
 
+
             self._log.info(f"Total jobs submitted: {len(job_list)}")
-            finished = cluster.parse_jobs(job_list, self._logging.getLogger("track.jobs"))  # track list of jobs
+            finished = cluster.parse_jobs(job_list, self._args.environment, self._logging.getLogger("track.jobs"))  #
+            # track list
+            # of jobs
 
             if finished:
+
                 self._log.info(f"Alignment jobs are finished!")
                 self._log.info(f"Merging alignment log files...")
                 # go through alignment output dir and parse all the log files
@@ -212,12 +232,25 @@ class fastq2counts(object):
         3. For each pair of sam files, submit jobs to the cluster
         4. Log to main log
         """
-        self._log.info("Counting mutations")
+        self._log.info("Counting mutations ...")
         # submit job with main.py -r1 and -r2
         # run main.py with -r1 and -r2
 
-        mut_counts = count_mut.readSam(self._r1, self._r2, self._param_json, self._args, self._output)
-        mut_counts._merged_main()
+        mut_counts = count_mut.readSam(self._r1, self._r2, self._param_json, self._args, self._output, self._args.c)
+        # start = time.time()
+        if self._args.test:
+            self._log.info("Testing... ")
+            # in testing mode, not using muticore
+            mut_counts._merged_main()
+        # end = time.time()
+        # print('Time taken for original program: ', end - start)
+        # testing multicore program
+        # start = time.time()
+        else:
+            self._log.info("Running multi-core analysis ... ")
+            mut_counts.multi_core()
+        # end = time.time()
+        # print('Time taken for 8 cores program: ', end - start)
 
     def _makejobs(self, sh_output, sam_dir):
         """
@@ -253,24 +286,28 @@ class fastq2counts(object):
             # submit job with main.py -r1 and -r2
             # run main.py with -r1 and -r2
             if self._args.sr_Override:
-                cmd = f"python {self._main_path} -n {self._args.name} -r1 {self._r1} -r2 {self._r2} -o {self._output} -p {self._param_json} --skip_alignment -log {self._args.log_level} -env {self._args.environment} -at {self._args.at} -mt {self._args.mt} -override"
+                cmd = f"tileseq_mut -n {self._args.name} -r1 {self._r1} -r2 {self._r2} -o {self._output} -p" \
+                      f" {self._param_json} --skip_alignment -log {self._args.log_level} -env {self._args.environment} -at {self._args.at} -mt {self._args.mt} -override"
             else:
-                cmd = f"python {self._main_path} -n {self._args.name} -r1 {self._r1} -r2 {self._r2} -o {self._output} -p {self._param_json} --skip_alignment -log {self._args.log_level} -env {self._args.environment} -at {self._args.at} -mt {self._args.mt}"
+                cmd = f"tileseq_mut -n {self._args.name} -r1 {self._r1} -r2 {self._r2} -o {self._output} -p {self._param_json} --skip_alignment -log {self._args.log_level} -env {self._args.environment} -at {self._args.at} -mt {self._args.mt}"
 
             if self._args.environment == "BC2" or self._args.environment == "BC":
                 logging.info("Submitting mutation counts jobs to BC2...")
-                job_id = cluster.mut_count_sh_bc(i, cmd, self._args.mt, sh_output, self._log)
+                job_id = cluster.mut_count_sh_bc(i, cmd, self._args.mt, sh_output, self._log, self._args.c)
             elif self._args.environment == "DC":
                 logging.info("Submitting mutation counts jobs to DC...")
                 # (sample_name, cmd, mt, sh_output_dir, logger)
-                job_id = cluster.mut_count_sh_dc(i, cmd, self._args.mt, sh_output, self._log) # this function will make a sh file for submitting the job
+                job_id = cluster.mut_count_sh_dc(i, cmd, self._args.mt, sh_output, self._log, self._args.c) # this
+                # function
+                # will make a sh file for submitting the job
 
             job_list.append(job_id)
 
         jobs = ",".join(job_list)
         self._log.debug(f"All jobs: {jobs}")
         self._log.info(f"Total jobs running: {len(job_list)}")
-        finished = cluster.parse_jobs(job_list, self._logging.getLogger("track.jobs")) # track list of jobs
+        finished = cluster.parse_jobs(job_list, self._args.environment, self._logging.getLogger("track.jobs")) # track
+        # list of jobs
 
         return finished
 
@@ -286,7 +323,6 @@ class fastq2counts(object):
                     mut_n += 1
         return mut_n
 
-
     def _main(self):
         """
         Main for the fastq2counts object
@@ -313,17 +349,23 @@ class fastq2counts(object):
 
             # output directory is the mut_count dir
             # make folder to store all the sh files
-            if self._args.environment == "BC2" or self._args.environment == "DC" or self._args.environment == "BC":
+            if self._args.environment == "BC2" or self._args.environment == "BC":
                 sh_output = os.path.join(self._output, "BC_mut_sh")
-                self._log.info(f"Mutation count sh files are made in {sh_output}")
-                os.mkdir(sh_output)
+
+            elif self._args.environment == "DC":
+                sh_output = os.path.join(self._output, "DC_mut_sh")
+
             else:
                 self._log.error("Please provide valid environment: BC/BC2/DC")
                 exit(1)
+
+            self._log.info(f"Mutation count sh files are made in {sh_output}")
+
+            os.mkdir(sh_output)
             finished = self._makejobs(sh_output, sam_dir)
 
             if finished:
-                self._log.info(f"Mutaion counting jobs are finished!")
+                self._log.info(f"Mutation counting jobs are finished!")
                 self._log.info("Check mutation counts file ...")
                 mutcount_list = glob.glob(os.path.join(self._output, "counts_sample_*.csv"))
                 self._log.info(f"{len(mutcount_list)} mutation counts file generated")
@@ -355,11 +397,11 @@ def check(args):
     # check if output dir exists
     if not os.path.isdir(args.output):
         print(f"Output directory not found: {args.output}")
-        exit(1)
+        exit(2)
     # check if fastq dir exists
     if args.fastq and not os.path.isdir(args.fastq):
         print(f"Fastq file path not found: {args.fastq}")
-        exit(1)
+        exit(3)
     # try convert csv to json in the same dir as the csv file
     # convert csv file to json
     if args.param.endswith(".csv"):
@@ -372,15 +414,17 @@ def check(args):
         else:
             convert = f"csv2json.R {args.param} -o {param_json}"
         os.system(convert)
+
     # if the file ends with .json, do nothing
     elif args.param.endswith(".json"):
         param_json = args.param
     else:
         print("Please provide valid paramter file format (csv or json)")
-        exit(1)
+        exit(4)
+
     if not os.path.isfile(param_json):
         print("Json file does not exist, check conversion!")
-        exit(1)
+        exit(5)
 
     return param_json
 
@@ -407,7 +451,8 @@ def main(args):
             if not os.path.isfile(param_path):
                 param_path = shutil.copy(param_json, args.output, follow_symlinks=True)
             # set up loggingthis creates main.log in the mut_count output dir
-            main_log = log(args.output, args.log_level)
+            main_log_f = os.path.join(args.output, "mutcount_main.log")
+            main_log = help_functions.logginginit(args.log_level, main_log_f)
 
             # initialize mutcount object
             mc = fastq2counts(param_path, args.output, main_log, args)
@@ -426,7 +471,8 @@ def main(args):
             param_path = os.path.join(updated_out, param_base)
             if not os.path.isfile(param_path):
                 param_path = shutil.copy(param_json, updated_out, follow_symlinks=True)
-            main_log = log(updated_out, args.log_level)
+            main_log_f = os.path.join(args.output, "mutcount_main.log")
+            main_log = help_functions.logginginit(args.log_level, main_log_f)
             # initialize mutcount object
             mc = fastq2counts(param_path, updated_out, main_log, args)
             mc._init_skip(skip=True)
@@ -442,7 +488,9 @@ def main(args):
         param_path = os.path.join(updated_out, param_base)
         if not os.path.isfile(param_path):
             param_path = shutil.copy(param_json, updated_out, follow_symlinks=True)
-        main_log = log(updated_out, args.log_level)
+
+        main_log_f = os.path.join(args.output, "main.log")
+        main_log = help_functions.logginginit(args.log_level, main_log_f)
 
         # initialize mutcount object
         mc = fastq2counts(param_path, updated_out, main_log, args)
@@ -460,29 +508,6 @@ def write_param(args_log_path, args):
             args_log.write(arg+",")
             args_log.write(f"{getattr(args, arg, 'N/A')}\n")
 
-def log(output_dir, log_level):
-    """
-    Make a logging object which writes to the main.log in output_dir
-    """
-    log_level = log_level.upper()
-    # init main log
-    logging.basicConfig(filename=os.path.join(output_dir, "main.log"),
-                    filemode="a",
-                    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
-                    datefmt="%m/%d/%Y %I:%M:%S %p",
-                    level = log_level)
-    # define a Handler which writes INFO messages or higher to the sys.stderr
-    console = logging.StreamHandler()
-    console.setLevel(log_level)
-    # set a format which is simpler for console use
-    formatter = logging.Formatter('%(asctime)s - %(name)-8s: %(levelname)-4s %(message)s')
-    # tell the handler to use this format
-    console.setFormatter(formatter)
-    # add the handler to the root logger
-    logging.getLogger('').addHandler(console)
-
-    return logging
-
 
 if __name__ == "__main__":
 
@@ -491,8 +516,8 @@ if __name__ == "__main__":
     parser.add_argument("-f", "--fastq", help="Path to all fastq files you want to analyze", type=str)
     parser.add_argument("-o", "--output", help="Output folder", type=str, required=True)
     parser.add_argument("-p", "--param", help="csv paramter file", type=str, required=True)
-    parser.add_argument("--skip_alignment", action="store_true", help="skip alignment for this analysis, ONLY submit jobs for counting mutations in existing output folder")
     parser.add_argument("-n", "--name", help="Name for this run", type=str, required=True)
+    parser.add_argument("--skip_alignment", action="store_true", help="skip alignment for this analysis, ONLY submit jobs for counting mutations in existing output folder")
     parser.add_argument("-r1", help="r1 SAM file", type=str)
     parser.add_argument("-r2", help="r2 SAM file", type=str)
 
@@ -504,7 +529,10 @@ if __name__ == "__main__":
     parser.add_argument("-at", type = int, help="Alignment time \
         (default = 8h)", default=8)
     parser.add_argument("-mt", type = int, help="Mutation call time \
-        (default = 36h)", default=36)
+        (default = 48h)", default=48)
+    parser.add_argument("-c", type=int, help="Number of cores", default=16)
+    parser.add_argument("-test", action="store_true", help="Turn on testing mode")
+    parser.add_argument("-rc", action="store_true", help="Turn on detect rc mode")
     ##parser.add_argument("-qual", "--quality", help="Posterior threshold for \
     ##    filtering mutations (default = 0.99)", type=float, default = 0.99)
     ##parser.add_argument("-min", "--min_cover", help="Minimal percentage required to \
